/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlCast.h"
#include "HvControlBinop.h"
#include "HvSignalVar.h"
#include "HvControlIf.h"
#include "HvSignalEnvelope.h"
#include "HvControlVar.h"
#include "HvMath.h"
#include "HvSignalLine.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_AOWh32aF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_7aZ9qUZL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xcMQvOGi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_FXtWZ8qa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_90jOk8jJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_kS1Mwy4o_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1DGNEOIY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_hgkCSjYX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Roa94WUn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Fl8avLkh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_wIo0VobJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Jz4wwxl4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_u2lU4bX3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_lGAlnPn3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_WMqY1MxC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_HlDXNIDq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_LwGL0Rsb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_TnRG3jMM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_VLOFP0b5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_neGnJuqk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_3SrpSFQ2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_mxEXMgxd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ojEuUgL9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_3LyBhLRM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_9aYUyr0Y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iuN9HNj6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_s0zWewfI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_DxL7XCU0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_oHSlmvoY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_XTqJAPbW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_hEDpZtzv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_9bgjT4cq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_82ZGIJ47_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_34vGFghB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_bW4CGVas_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_mjEjMZXR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jSBEhVnV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_8g86bM9V_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_J6ZnqaKE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LFocpjcO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_3J5pssO7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_2lrTLdfi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_b6OlBRd5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_ELs2FYDf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_KOvLJyT0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_a2OrsxJ8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_LkNCOM90_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Zd1vE7p4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_y5AhQlgo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ydMaEqZO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_dtmIBvD3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_USZMRuwE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_6XnfnclD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Thd8E16u_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_5QyMtSoS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_JUxmGCGo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_BPJzdzi0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_3nyV1GSF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_RboHcdp9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_dnxmXog4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_3mCJzJn8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_XVAKs4b5_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_jQWTLzmv;
  SignalEnvelope sEnv_TnRG3jMM;
  SignalEnvelope sEnv_VLOFP0b5;
  SignalEnvelope sEnv_bW4CGVas;
  SignalEnvelope sEnv_mjEjMZXR;
  SignalLine sLine_aFz6mEpw;
  SignalLine sLine_lEkC9hDi;
  ControlBinop cBinop_AOWh32aF;
  ControlBinop cBinop_7aZ9qUZL;
  ControlBinop cBinop_xcMQvOGi;
  ControlBinop cBinop_FXtWZ8qa;
  ControlBinop cBinop_1DGNEOIY;
  ControlBinop cBinop_Roa94WUn;
  ControlBinop cBinop_Fl8avLkh;
  ControlBinop cBinop_wIo0VobJ;
  ControlVar cVar_Jz4wwxl4;
  ControlVar cVar_lGAlnPn3;
  SignalVarf sVarf_ttU9i1IA;
  ControlVar cVar_WMqY1MxC;
  ControlIf cIf_HlDXNIDq;
  ControlIf cIf_LwGL0Rsb;
  ControlBinop cBinop_neGnJuqk;
  ControlBinop cBinop_mxEXMgxd;
  ControlBinop cBinop_ojEuUgL9;
  ControlBinop cBinop_iuN9HNj6;
  ControlBinop cBinop_DxL7XCU0;
  SignalVarf sVarf_EOQGUuoV;
  ControlVar cVar_9bgjT4cq;
  ControlIf cIf_82ZGIJ47;
  ControlIf cIf_34vGFghB;
  ControlBinop cBinop_jSBEhVnV;
  ControlBinop cBinop_J6ZnqaKE;
  ControlBinop cBinop_LFocpjcO;
  ControlBinop cBinop_b6OlBRd5;
  ControlBinop cBinop_KOvLJyT0;
  ControlVar cVar_y5AhQlgo;
  ControlBinop cBinop_ydMaEqZO;
  SignalVarf sVarf_e7qbv2QE;
  SignalVarf sVarf_Ti453Pp8;
  SignalVarf sVarf_msS7IOjd;
  SignalVarf sVarf_JL8bv6h3;
  ControlBinop cBinop_Thd8E16u;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
